@@ -1,10 +1,10 @@
-# Copyright (c) 2003-2006 CORE Security Technologies
+# Copyright (c) 2003-2012 CORE Security Technologies
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: structure.py,v 1.2 2006/05/23 21:19:26 gera Exp $
+# $Id: structure.py 882 2013-11-01 20:27:45Z bethus@gmail.com $
 #
 
 from struct import pack, unpack, calcsize
@@ -23,7 +23,6 @@ class Structure:
         format specifiers:
           specifiers from module pack can be used with the same format 
           see struct.__doc__ (pack/unpack is finally called)
-            >       [little endian]
             x       [padding byte]
             c       [character]
             b       [signed byte]
@@ -35,7 +34,7 @@ class Structure:
             i       [signed integer]
             I       [unsigned integer]
             q       [signed long long (quad)]
-            Q       [unsigned long ong (quad)]
+            Q       [unsigned long long (quad)]
             s       [string (array of chars), must be preceded with length in format specifier, padded with zeros]
             p       [pascal string (includes byte count), must be preceded with length in format specifier, padded with zeros]
             f       [float]
@@ -51,7 +50,7 @@ class Structure:
 
             %08x    will output an 8 bytes hex
             %s      will output a string
-            %s\x00  will output a NUL terminated string
+            %s\\x00  will output a NUL terminated string
             %d%d    will output 2 decimal digits (against the very same specification of Structure)
             ...
 
@@ -59,7 +58,7 @@ class Structure:
             :       just copy the bytes from the field into the output string (input may be string, other structure, or anything responding to __str__()) (for unpacking, all what's left is returned)
             z       same as :, but adds a NUL byte at the end (asciiz) (for unpacking the first NUL byte is used as terminator)  [asciiz string]
             u       same as z, but adds two NUL bytes at the end (after padding to an even size with NULs). (same for unpacking) [unicode string]
-            w       DCE-RPC/NDR string (it's a macro for [  '<L=(len(field)+1)/2','"\x00\x00\x00\x00','<L=(len(field)+1)/2',':' ]
+            w       DCE-RPC/NDR string (it's a macro for [  '<L=(len(field)+1)/2','"\\x00\\x00\\x00\\x00','<L=(len(field)+1)/2',':' ]
             ?-field length of field named 'field', formated as specified with ? ('?' may be '!H' for example). The input value overrides the real length
             ?1*?2   array of elements. Each formated as '?2', the number of elements in the array is stored as specified by '?1' (?1 is optional, or can also be a constant (number), for unpacking)
             'xxxx   literal xxxx (field's value doesn't change the output. quotes must not be closed or escaped)
@@ -80,11 +79,18 @@ class Structure:
             self.alignment = alignment
 
         self.fields    = {}
+        self.rawData   = data
         if data is not None:
             self.fromString(data)
         else:
             self.data = None
 
+    @classmethod
+    def fromFile(self, file):
+        answer = self()
+        answer.fromString(file.read(len(answer)))
+        return answer
+
     def setAlignment(self, alignment):
         self.alignment = alignment
 
@@ -129,10 +135,13 @@ class Structure:
         return data
 
     def fromString(self, data):
+        self.rawData = data
         for field in self.commonHdr+self.structure:
             if self.debug:
                 print "fromString( %s | %s | %r )" % (field[0], field[1], data)
             size = self.calcUnpackSize(field[1], data, field[0])
+            if self.debug:
+                print "  size = %d" % size
             dataClassOrCode = str
             if len(field) > 2:
                 dataClassOrCode = field[2]
@@ -183,6 +192,16 @@ class Structure:
         if format[:1] == "'" or format[:1] == '"':
             return format[1:]
 
+        # code specifier
+        two = format.split('=')
+        if len(two) >= 2:
+            try:
+                return self.pack(two[0], data)
+            except:
+                fields = {'self':self}
+                fields.update(self.fields)
+                return self.pack(two[0], eval(two[1], {}, fields))
+
         # address specifier
         two = format.split('&')
         if len(two) == 2:
@@ -190,18 +209,10 @@ class Structure:
                 return self.pack(two[0], data)
             except:
                 if (self.fields.has_key(two[1])) and (self[two[1]] is not None):
-                    return self.pack(two[0], id(self[two[1]]))
+                    return self.pack(two[0], id(self[two[1]]) & ((1<<(calcsize(two[0])*8))-1) )
                 else:
                     return self.pack(two[0], 0)
 
-        # code specifier
-        two = format.split('=')
-        if len(two) >= 2:
-            try:
-                return self.pack(two[0], data)
-            except:
-                return self.pack(two[0], eval(two[1], {}, self.fields))
-
         # length specifier
         two = format.split('-')
         if len(two) == 2:
@@ -269,7 +280,11 @@ class Structure:
         # void specifier
         if format[:1] == '_':
             if dataClassOrCode != str:
-                return eval(dataClassOrCode, {}, self.fields)
+                fields = {'self':self, 'inputDataLeft':data}
+                fields.update(self.fields)
+                return eval(dataClassOrCode, {}, fields)
+            else:
+                return None
 
         # quote specifier
         if format[:1] == "'" or format[:1] == '"':
@@ -417,6 +432,10 @@ class Structure:
         if self.debug:
             print "  calcUnpackSize( %s | %s | %r)" %  (field, format, data)
 
+        # void specifier
+        if format[:1] == '_':
+            return 0
+
         addressField = self.findAddressFieldFor(field)
         if addressField is not None:
             if not self[addressField]:
@@ -430,10 +449,6 @@ class Structure:
 
         # XXX: Try to match to actual values, raise if no match
         
-        # void specifier
-        if format[:1] == '_':
-            return 0
-
         # quote specifier
         if format[:1] == "'" or format[:1] == '"':
             return len(format)-1
@@ -542,17 +557,32 @@ class Structure:
         for field in self.commonHdr + self.structure:
             self[field[0]] = self.zeroValue(field[1])
 
-    def dump(self, msg, indent = 0):
+    def dump(self, msg = None, indent = 0):
         import types
+        if msg is None: msg = self.__class__.__name__
         ind = ' '*indent
         print "\n%s" % (msg)
-        for i in self.fields.keys():
+        fixedFields = []
+        for field in self.commonHdr+self.structure:
+            i = field[0] 
+            if i in self.fields:
+                fixedFields.append(i)
+                if isinstance(self[i], Structure):
+                    self[i].dump('%s%s:{' % (ind,i), indent = indent + 4)
+                    print "%s}" % ind
+                else:
+                    print "%s%s: {%r}" % (ind,i,self[i])
+        # Do we have remaining fields not defined in the structures? let's 
+        # print them
+        remainingFields = list(set(self.fields) - set(fixedFields))
+        for i in remainingFields:
             if isinstance(self[i], Structure):
-                self[i].dump('%s:{' % i, indent = indent + 4)
-                print "}"
+                self[i].dump('%s%s:{' % (ind,i), indent = indent + 4)
+                print "%s}" % ind
             else:
                 print "%s%s: {%r}" % (ind,i,self[i])
 
+
 class _StructureTest:
     alignment = 0
     def create(self,data = None):
@@ -677,6 +707,28 @@ class _Test_UnpackCode(_StructureTest):
         a['uno'] = 'soy un loco!'
         a['dos'] = 'que haces fiera'
 
+class _Test_AAA(_StructureTest):
+    class theClass(Structure):
+        commonHdr = ()
+        structure = (
+          ('iv', '!L=((init_vector & 0xFFFFFF) << 8) | ((pad & 0x3f) << 2) | (keyid & 3)'),
+          ('init_vector',   '_','(iv >> 8)'),
+          ('pad',           '_','((iv >>2) & 0x3F)'),
+          ('keyid',         '_','( iv & 0x03 )'),
+          ('dataLen',       '_-data', 'len(inputDataLeft)-4'),
+          ('data',':'),
+          ('icv','>L'),
+        )
+
+    def populate(self, a):
+        a['init_vector']=0x01020304
+        #a['pad']=int('01010101',2)
+        a['pad']=int('010101',2)
+        a['keyid']=0x07
+        a['data']="\xA0\xA1\xA2\xA3\xA4\xA5\xA6\xA7\xA8\xA9"
+        a['icv'] = 0x05060708
+        #a['iv'] = 0x01020304
+        
 if __name__ == '__main__':
     _Test_simple().run()
 
@@ -691,3 +743,4 @@ if __name__ == '__main__':
     _Test_Optional_sparse().run()
     _Test_AsciiZArray().run()
     _Test_UnpackCode().run()
+    _Test_AAA().run()
